import Popper from 'popper.js';

const PopperMixin = {
  props: {
    always: {
      type: Boolean,
      default: false,
    },
    trigger: {
      type: String,
      default: 'hover',
    },
    appendToBody: {
      type: Boolean,
      default: true,
    },
    content: {
      type: String,
      default: '',
    },
    placement: {
      type: String,
      default: 'top',
    },
    disabled: {
      type: Boolean,
      default: false,
    },
  },

  data() {
    return {
      reference: null,
      popper: null,
      isShow: false,
    };
  },

  watch: {
    disabled(val) {
      if (!val) {
        this.runPopper();
      } else {
        this.destroy();
      }
    },
  },
  methods: {
    // add delay
    hidePopper(popperEl) {
      if(this.popper){
        popperEl.style.display="none";
        this.popper.destroy(); // destroy popper when hide
        this.popper = null;
      }
    },
    showPopper(popperEl) {
      popperEl.style.display="block";
      if (this.timer) clearTimeout(this.timer);
      if (this.popperTimer) clearTimeout(this.popperTimer);
    },

    createInstance(reference,popperEl,options) {
      this.showPopper(popperEl);
      if (this.popper) {
        this.popper.update();
        return;
      }
      if (this.appendToBody) document.body.appendChild(popperEl);
      this.popper = new Popper(reference, popperEl, options);
    },
    runPopper(reference,popperEl,options) {
      var vueObj = this;
      this.destroy();
      if (this.disabled) return;
      if (this.always) {
        this.createInstance(reference,popperEl,options);
      } else {
        this.createInstance(reference,popperEl,options);
      }
      document.documentElement.removeEventListener('click',vueObj.hidePopper);
      document.documentElement.addEventListener('click', function(){
        vueObj.hidePopper(popperEl);
      });
    },

    destroy() {
      if (this.popper) {
        this.popper.destroy();
        this.popper = null;
      }
    },
  },
  mounted() {
    // todo disabled,
    // this.runPopper();
    // console.log("开始啦");
  },

  beforeDestroy() {
    if(this.$refs.popper){
      this.$refs.popper.remove();
    }
    this.destroy();
  },
};

export default PopperMixin;
